/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* longopts.c - parse the command line for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

static char short_ops[] = "D:r:s:d:cCaAb:pPoOv:gGfFhV";

static struct option long_ops[] = {

	{"alsa-device", required_argument, 0, 'D'},
	{"sample-rate", required_argument, 0, 'r'},
	{"stereo-channel", required_argument, 0, 's'},
	{"dc-offset", required_argument, 0, 'd'},
	{"rainbow-mode", no_argument, 0, 'c'},
	{"no-rainbow-mode", no_argument, 0, 'C'},
	{"averaging-mode", no_argument, 0, 'a'},
	{"no-averaging-mode", no_argument, 0, 'A'},
	{"averaging-factor", required_argument, 0, 'b'},
	{"peak-mode", no_argument, 0, 'p'},
	{"no-peak-mode", no_argument, 0, 'P'},
	{"overlap-mode", no_argument, 0, 'o'},
	{"no-overlap-mode", no_argument, 0, 'O'},
	{"overlap-factor", required_argument, 0, 'v'},
	{"graticule-mode", no_argument, 0, 'g'},
	{"no-graticule-mode", no_argument, 0, 'G'},
	{"freeze-mode", no_argument, 0, 'f'},
	{"no-freeze-mode", no_argument, 0, 'F'},
	{"help", no_argument, 0, 'h'},
	{"version", no_argument, 0, 'V'},
	{0, 0, 0, 0}
};

void
parse_command_line (int argc, char *argv[])
{
	int c;

	while ((c = getopt_long (argc, argv, short_ops, long_ops, NULL)) != -1)
	{
		switch (c)
		{
			case 'D':								/* ALSA device */

				strncpy (alsa_device, optarg, ALSA_LEN - 1);
				alsa_device[ALSA_LEN - 1] = 0;
				break;

			case 'r':								/* Sampling rate */

				rate_index = strtol (optarg, NULL, 10);
				break;

			case 's':								/* Stereo mode */

				stereo_channel = strtol (optarg, NULL, 10);
				break;

			case 'd':								/* DC offset */

				dc_offset = strtol (optarg, NULL, 10);
				break;

			case 'c':								/* Rainbow mode on */

				rainbow_mode = 1;
				break;

			case 'C':								/* Rainbow mode off */

				rainbow_mode = 0;
				break;

			case 'a':								/* Averaging mode on */

				averaging_mode = 1;
				break;

			case 'A':								/* Averaging mode off */

				averaging_mode = 0;
				break;

			case 'b':								/* Averaging factor */

				averaging_factor = strtol (optarg, NULL, 10);
				break;

			case 'p':								/* Peak mode on */

				peak_mode = 1;
				break;

			case 'P':								/* Peak mode off */

				peak_mode = 0;
				break;

			case 'o':								/* Overlap mode on */

				overlap_mode = 1;
				break;

			case 'O':								/* Overlap mode off */

				overlap_mode = 0;
				break;

			case 'v':								/* Overlap factor */

				overlap_factor = strtol (optarg, NULL, 10);
				break;

			case 'g':								/* Graticule mode on */

				graticule_mode = 1;
				break;

			case 'G':								/* Graticule mode off */

				graticule_mode = 0;
				break;

			case 'f':								/* Freeze mode on */

				freeze_mode = 1;
				break;

			case 'F':								/* Freeze mode off */

				freeze_mode = 0;
				break;

			case 'h':								/* Help information */

				show_help ();
				break;

			case 'V':								/* Version information */

				show_version ();
				break;

			default:									/* None of the above ... */

				show_error ();
				break;

		}														/* end switch */
	}															/* end while */
}																/* end parse_command_line */

void
show_help (void)
{
	int count, *ip = allow_rates;
	char **cp1 = mode_names;

	puts ("\nUsage : " PROGNAME " [OPTIONS]\n");
	puts ("All parameters N are integers\n");

	puts ("  -D, --alsa-device=<string>  ALSA device\n");
	puts ("  -r, --sample-rate=N         sampling rate\n");

	for (count = 0; count <= MAXINDEX; count++, ip++)
		printf ("                              %d   (%5d Hz)\n", count, *ip);

	puts ("");
	puts ("  -s, --stereo-channel=N      stereo channel.\n");

	for (count = 0; count <= STEREOMAX; count++, cp1++)
		printf ("                              %d   (%s)\n", count, *cp1);

	puts ("");
	printf ("  -d, --dc-offset=N           "
					"DC offset (%d to %d).\n\n", MINDC, MAXDC);

	puts ("  -c, --rainbow-mode          rainbow mode on");
	puts ("  -C, --no-rainbow-mode       rainbow mode off\n");

	puts ("  -a, --averaging-mode        averaging mode on");
	puts ("  -A, --no-averaging-mode     averaging mode off");

	printf ("  -b, --averaging-factor=N    "
					"averaging factor %% (%d to %d)\n\n", AVERMIN, AVERMAX);

	puts ("  -p, --peak-mode             peak mode on");
	puts ("  -P, --no-peak-mode          peak mode off\n");

	puts ("  -o, --overlap-mode          overlap mode on");
	puts ("  -O, --no-overlap-mode       overlap mode off");

	printf ("  -v, --overlap-factor=N      "
					"overlap factor (%d to %d)\n\n", OVERMIN, OVERMAX);

	puts ("  -g, --graticule-mode        graticule mode on");
	puts ("  -G, --no-graticule-mode     graticule mode off\n");

	puts ("  -f, --freeze-mode           freeze mode on");
	puts ("  -F, --no-freeze-mode        freeze mode off\n");

	puts ("  -h, --help                  this message");
	puts ("  -V, --version               version information\n");
	exit (EXIT_FAILURE);

}																/* end show_help */

void
show_version (void)
{
	puts (PROGNAME " " VERSION);
	exit (EXIT_FAILURE);

}																/* end show_version */

void
show_error (void)
{
	puts ("Enter `" PROGNAME " --help' for help on options.");
	exit (EXIT_FAILURE);

}																/* end show_error */

/* vi: set ts=2: */
