/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* sound.c - routines for soundcards in `xspect' by Gordon Miller 2007 */

#define ALSA_PCM_NEW_HW_PARAMS_API

#include "xspect.h"
#include "extern.h"

static snd_pcm_t *handle;

void
init_record (void)
/* Called every time soundcard is started ... */
{
	snd_pcm_hw_params_t *params;
	unsigned int sampling_rate;

/* Open soundcard for business ... */
	if (snd_pcm_open (&handle, alsa_device, SND_PCM_STREAM_CAPTURE, 0) < 0)
		error ("Unable to open the soundcard.");

/* Allocate a hardware parameters object. */
	if (snd_pcm_hw_params_malloc (&params) < 0)
		error ("Unable to allocate space for soundcard parameters.");

/* Fill it in with default values. */
	if (snd_pcm_hw_params_any (handle, params) < 0)
		error ("Unable to initialize soundcard parameters.");

/* Interleaved mode */
	if (snd_pcm_hw_params_set_access
			(handle, params, SND_PCM_ACCESS_RW_INTERLEAVED) < 0)
		error ("Cannot set soundcard access type.");

/* Set 16 bit little-endian sample format ... */
	if (snd_pcm_hw_params_set_format
			(handle, params, SND_PCM_FORMAT_S16_LE) < 0)
		error ("Cannot set soundcard sample format.");

/* Set mono/stereo ... */
	if (snd_pcm_hw_params_set_channels
			(handle, params, stereo_channel ? 2 : 1) < 0)
		error ("Cannot set soundcard channels.");

/* Set sampling rate ... */
	sampling_rate = allow_rates[rate_index];
	if (snd_pcm_hw_params_set_rate_near
			(handle, params, &sampling_rate, NULL) < 0)
		error ("Cannot set soundcard sampling rate.");

/* And make the changes ... */
	if (snd_pcm_hw_params (handle, params) < 0)
		error ("Unable to set soundcard parameters.");

/* Free up params ... */
	snd_pcm_hw_params_free (params);

/* Soundcard is up & ready ... */
	sound_set = 1;

/* Get first block with new settings ... */
	if (overlap_mode)
	{
		record_block (buffer[queue_buffer]);
		queue_buffer = !queue_buffer;
	}

	else
		record_block (buffer[0]);

}																/* end init_record */

void
reset_soundcard (void)
/* Called every time soundcard is stopped ... */
{
	snd_pcm_drop (handle);
	snd_pcm_close (handle);
	sound_set = 0;

}																/* end reset_soundcard */

void
callibrate_soundcard (void)
/* Calculate DC error by averaging all samples ... */
{
	int i, total = 0, remainder;
	short *s16 = buffer[0];

	if (stereo_channel == 0)
	/* Mono selected ... */
		for (i = MAX_LEN; i; i--, s16++)
			total += *s16;

	else if (stereo_channel == 1)
	/* Left channel selected ... */
		for (i = MAX_LEN; i; i--, s16 += 2)
			total += *s16;

	else
	/* Right channel selected ... */
		for (s16++, i = MAX_LEN; i; i--, s16 += 2)
			total += *s16;

/* Integer arithmetic ... */
	remainder = total % MAX_LEN;
	dc_offset = total / MAX_LEN;

	if (remainder > MAX_LEN / 2)
		dc_offset++;

/* Clamp to min and max allowed values ... */
	if (dc_offset < MINDC)
		dc_offset = MINDC;

	if (dc_offset > MAXDC)
		dc_offset = MAXDC;

}																/* end callibrate */

void
record_block (void *buffer)
{
	int rc, needed = MAX_LEN;
	void *buf_ptr = buffer;

	do
	{
	/* Try to read soundcard ... */
		rc = snd_pcm_readi (handle, buf_ptr, needed);

		if (rc == -EAGAIN || (rc >= 0 && rc < needed))
		/* Not enough samples read ... */
		{
		/* Try to wait for soundcard to be ready ... */
			if (snd_pcm_wait (handle, 1000) < 0)
				error ("Cannot read soundcard.");
		}

		else if (rc == -EPIPE)
		/* An overrun ... */
		{
		/* Try to re-initialize soundcard ... */
			if (snd_pcm_prepare (handle) < 0)
				error ("Cannot prepare soundcard.");
		}

		else if (rc == -ESTRPIPE)
		/* Soundcard suspended ... */
		{
			int res;

		/* Try to unsuspend soundcard ... */
			while ((res = snd_pcm_resume (handle)) == -EAGAIN)
				sleep (1);

		/* No dice ... */
			if (res < 0)
			/* Try to re-initialize soundcard ... */
				if (snd_pcm_prepare (handle) < 0)
					error ("Cannot prepare soundcard.");
		}

		else if (rc < 0)
		/* This shouldn't happen ... */
		{
			error ("Cannot read soundcard.");
		}

		if (rc > 0)
		/* If we read something, adjust the count & pointer */
		{
			needed -= rc;
			buf_ptr += rc * (stereo_channel ? BPSSTEREO : BPSMONO);
/*			printf ("needed : %d, read = %d\n", needed, rc);*/
		}
	}
	while (needed);
/* Keep going until buffer filled ... */

}																/* end record_block */

void
init_frame_rate (void)
/* Set the screen refresh rate and dialog latency ... */
{
	frame_rate = (double) allow_rates[rate_index] *
		(overlap_mode ? overlap_factor : 1) / MAX_LEN;

	frame_period = TPS / frame_rate;

}																/* end init_frame_rate */

/* vi: set ts=2: */
