/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* xgraphics.c - graphics functions for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

static int screen;
static Colormap colourmap;

void
setup_graphics (void)
{
	XGCValues values;
	Font font1, font2;
	XSizeHints *sizehints;

/* initialise Xlib threads support */
	if (XInitThreads () == 0)
		error ("Cannot initialise Xlib threads support.");

/* Try to open display */
	if ((display = XOpenDisplay (NULL)) == 0)
		error ("Cannot open X connection.");

	graph_set = 1;

/* Set window parameters */
	screen = DefaultScreen (display);

	if (DefaultVisual (display, screen)->class != TrueColor)
		error ("Can only run in TrueColor.");

	depth = DefaultDepth (display, screen);
	colourmap = DefaultColormap (display, screen);

/* Set up a blank window */
	window = XCreateSimpleWindow (display, DefaultRootWindow (display),
																X, Y, W, H, 0, 0, 0);

/* Create the shadow pixmap ... */
	shadow = XCreatePixmap (display, window, W, H, depth);

/* Make shadow pixmap the window background pixmap ... */
	XSetWindowBackgroundPixmap (display, window, shadow);

/* Allocate space for hints */
	if ((sizehints = XAllocSizeHints ()) == 0)
		error ("Not enough memory for the size hints structure.");

/* Don't allow resize ... and place according to program */
	sizehints->flags = PPosition | PSize | PMinSize | PMaxSize;

	sizehints->min_width = W;
	sizehints->min_height = H;
	sizehints->max_width = W;
	sizehints->max_height = H;

/* Set window name, icon name & sizehints */
	XSetStandardProperties (display, window, WNAME, WNAME, 0, 0, 0, sizehints);

/* Free up the hints space */
	XFree (sizehints);

/* Setup for ICCCM delete window. */
	wm_protocols = XInternAtom (display, "WM_PROTOCOLS", False);
	wm_delete_window = XInternAtom (display, "WM_DELETE_WINDOW", False);
	XSetWMProtocols (display, window, &wm_delete_window, 1);

/* Set the GCs for drawing */
	values.function = GXxor;
	graph_gc = XCreateGC (display, shadow, GCFunction, &values);

	values.function = GXxor;
	rain_gc = XCreateGC (display, shadow, GCFunction, &values);

	values.function = GXxor;
	grat_gc = XCreateGC (display, shadow, GCFunction, &values);

	back_gc = XCreateGC (display, shadow, 0, &values);
	border_gc = XCreateGC (display, shadow, 0, &values);
	gback_gc = XCreateGC (display, shadow, 0, &values);

/* Set text GCs */
	text_gc = XCreateGC (display, shadow, 0, &values);
	high_gc = XCreateGC (display, shadow, 0, &values);
	label_gc = XCreateGC (display, shadow, 0, &values);
	low_gc = XCreateGC (display, shadow, 0, &values);

/* Set menu GCs */
	dialog_fore_gc = XCreateGC (display, shadow, 0, &values);
	dialog_back_gc = XCreateGC (display, shadow, 0, &values);
	dialog_title_gc = XCreateGC (display, shadow, 0, &values);
	dialog_hl_black_gc = XCreateGC (display, shadow, 0, &values);
	dialog_hl_white_gc = XCreateGC (display, shadow, 0, &values);
	dialog_dim_gc = XCreateGC (display, shadow, 0, &values);
	dialog_high_gc = XCreateGC (display, shadow, 0, &values);

/* Set fonts */
	font1 = XLoadFont (display, FONT1);
	font2 = XLoadFont (display, FONT2);
	XSetFont (display, text_gc, font1);
	XSetFont (display, label_gc, font1);
	XSetFont (display, high_gc, font2);
	XSetFont (display, low_gc, font2);
	XSetFont (display, dialog_fore_gc, font1);
	XSetFont (display, dialog_title_gc, font1);
	XSetFont (display, dialog_dim_gc, font1);
	XSetFont (display, dialog_high_gc, font2);

/* Catch key & button events only */
	XSelectInput (display, window, KeyPressMask | ButtonPressMask);

/* Fire up the window */
	XMapRaised (display, window);

}																/* end setup_graphics */

void
cleanup_graphics (void)
{
	XCloseDisplay (display);

}																/* end cleanup_graphics */

void
erase_canvas (void)
{
	XFillRectangle (display, shadow, back_gc, 0, 0, W, H);

}																/* end erase_canvas */

void
setup_palette (void)
{
/* Main backgound, also used for changing text GC backgounds */
	unsigned long back_pixel = draw_setgc_fore (back_gc, &back);

/* Fixed black and white colours */
	XSetForeground (display, gback_gc, BlackPixel (display, screen));
	XSetForeground (display, dialog_hl_black_gc, BlackPixel (display, screen));
	XSetForeground (display, dialog_hl_white_gc, WhitePixel (display, screen));

	draw_setgc_fore (border_gc, &border);
	draw_setgc_fore (graph_gc, &graph);
	draw_setgc_fore (grat_gc, &grat);
	draw_setgc_fore (text_gc, &text);

/* Changing text GCs */
	draw_setgc_fore (label_gc, &label);
	draw_setgc_back (label_gc, back_pixel);

	draw_setgc_fore (high_gc, &high);
	draw_setgc_back (high_gc, back_pixel);

	draw_setgc_fore (low_gc, &low);
	draw_setgc_back (low_gc, back_pixel);

/* Menu GCs */
	draw_setgc_fore (dialog_fore_gc, &dialog_fore);
	draw_setgc_fore (dialog_back_gc, &dialog_back);
	draw_setgc_fore (dialog_title_gc, &dialog_title);
	draw_setgc_fore (dialog_dim_gc, &dialog_dim);
	draw_setgc_fore (dialog_high_gc, &dialog_high);

}																/* end setup_palette */

unsigned long
draw_setgc_fore (GC gc, Rgb * colour)
{
	XColor x_colour;

	x_colour.flags = DoRed | DoGreen | DoBlue;
	x_colour.red = colour->red * 1024;
	x_colour.green = colour->green * 1024;
	x_colour.blue = colour->blue * 1024;
	XAllocColor (display, colourmap, &x_colour);
	XSetForeground (display, gc, x_colour.pixel);
	return x_colour.pixel;

}																/* end draw_setgc_fore */

void
draw_setgc_back (GC gc, unsigned long pixel)
{
	XSetBackground (display, gc, pixel);

}																/* end draw_setgc_back */

/* vi: set ts=2: */
